@@ -98,17 +98,11 @@ KisTileDataStore* KisTileDataStore::instance()
 
 KisTileDataStore::MemoryStatistics KisTileDataStore::memoryStatistics()
 {
-    // in case the pooler is disabled, we should force it
-    // to update the stats
-    if (!m_pooler.isRunning()) {
-        m_pooler.forceUpdateMemoryStats();
-    }
-
     QReadLocker lock(&m_iteratorLock);
 
     MemoryStatistics stats;
 
-    const qint64 metricCoeff = KisTileData::WIDTH * KisTileData::HEIGHT;
+    const qint64 metricCoeff = qint64(KisTileData::WIDTH) * KisTileData::HEIGHT;
 
     stats.realMemorySize = m_pooler.lastRealMemoryMetric() * metricCoeff;
     stats.historicalMemorySize = m_pooler.lastHistoricalMemoryMetric() * metricCoeff;
@@ -121,11 +115,27 @@ KisTileDataStore::MemoryStatistics KisTileDataStore::memoryStatistics()
     return stats;
 }
 
+void KisTileDataStore::tryForceUpdateMemoryStatisticsWhileIdle()
+{
+    // in case the pooler is disabled, we should force it
+    // to update the stats
+    if (!m_pooler.isRunning()) {
+        m_pooler.forceUpdateMemoryStats();
+    }
+}
+
 inline void KisTileDataStore::registerTileDataImp(KisTileData *td)
 {
     int index = m_counter.fetchAndAddOrdered(1);
     td->m_tileNumber = index;
+
+    // make sure that access to the hash table is guarded by GC block
+    // (it avoids removal of the referenced cells caused by concurrent
+    // migrations)
+    m_tileDataMap.getGC().lockRawPointerAccess();
     m_tileDataMap.assign(index, td);
+    m_tileDataMap.getGC().unlockRawPointerAccess();
+
     m_numTiles.ref();
     m_memoryMetric += td->pixelSize();
 }
@@ -138,6 +148,11 @@ void KisTileDataStore::registerTileData(KisTileData *td)
 
 inline void KisTileDataStore::unregisterTileDataImp(KisTileData *td)
 {
+    // make sure that access to the hash table is guarded by GC block
+    // (it avoids removal of the referenced cells caused by concurrent
+    // migrations)
+    m_tileDataMap.getGC().lockRawPointerAccess();
+
     if (m_clockIndex == td->m_tileNumber) {
         do {
             m_clockIndex.ref();
@@ -149,6 +164,8 @@ inline void KisTileDataStore::unregisterTileDataImp(KisTileData *td)
     m_tileDataMap.erase(index);
     m_numTiles.deref();
     m_memoryMetric -= td->pixelSize();
+
+    m_tileDataMap.getGC().unlockRawPointerAccess();
 }
 
 void KisTileDataStore::unregisterTileData(KisTileData *td)
@@ -262,12 +279,9 @@ bool KisTileDataStore::trySwapTileData(KisTileData *td)
     if (!td->m_swapLock.tryLockForWrite()) return result;
 
     if (td->data()) {
-        unregisterTileDataImp(td);
         if (m_swappedStore.trySwapOutTileData(td)) {
+            unregisterTileDataImp(td);
             result = true;
-        } else {
-            result = false;
-            registerTileDataImp(td);
         }
     }
     td->m_swapLock.unlock();
