@@ -83,6 +83,9 @@ public:
      */
     ~KisMainWindow() override;
 
+    QMenu *createPopupMenu() override;
+
+
     QUuid id() const;
 
     /**
@@ -495,7 +498,7 @@ private:
      */
     QDockWidget* createDockWidget(KoDockFactoryBase* factory);
 
-    bool openDocumentInternal(const QUrl &url, KisMainWindow::OpenFlags flags = 0);
+    bool openDocumentInternal(const QUrl &url, KisMainWindow::OpenFlags f = KisMainWindow::OpenFlags());
 
     void updateReloadFileAction(KisDocument *doc);
 
