@@ -8,7 +8,7 @@
  *                2003-2011 Boudewijn Rempt <boud@valdyas.org>
  *                2004 Clarence Dang <dang@kde.org>
  *                2011 José Luis Vergara <pentalis@gmail.com>
- *                2017 L. E. Segovia <leo.segovia@siggraph.org>
+ *                2017 L. E. Segovia <amy@amyspark.me>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -348,6 +348,7 @@ void KisViewManager::initializeResourceManager(KoCanvasResourceProvider *resourc
     resourceManager->addDerivedResourceConverter(toQShared(new KisLodSizeThresholdResourceConverter));
     resourceManager->addDerivedResourceConverter(toQShared(new KisLodSizeThresholdSupportedResourceConverter));
     resourceManager->addDerivedResourceConverter(toQShared(new KisEraserModeResourceConverter));
+    resourceManager->addDerivedResourceConverter(toQShared(new KisPatternSizeResourceConverter));
     resourceManager->addResourceUpdateMediator(toQShared(new KisPresetUpdateMediator));
 }
 
@@ -360,9 +361,8 @@ void KisViewManager::slotViewAdded(KisView *view)
 {
     // WARNING: this slot is called even when a view from another main windows is added!
     //          Don't expect \p view be a child of this view manager!
-    Q_UNUSED(view);
 
-    if (viewCount() == 0) {
+    if (view->viewManager() == this && viewCount() == 0) {
         d->statusBar.showAllStatusBarItems();
     }
 }
@@ -371,9 +371,8 @@ void KisViewManager::slotViewRemoved(KisView *view)
 {
     // WARNING: this slot is called even when a view from another main windows is removed!
     //          Don't expect \p view be a child of this view manager!
-    Q_UNUSED(view);
 
-    if (viewCount() == 0) {
+    if (view->viewManager() == this && viewCount() == 0) {
         d->statusBar.hideAllStatusBarItems();
     }
 
@@ -501,6 +500,7 @@ void KisViewManager::setCurrentView(KisView *view)
         d->currentImageView->notifyCurrentStateChanged(true);
         d->currentImageView->canvasController()->activate();
         d->currentImageView->canvasController()->setFocus();
+        d->currentImageView->zoomManager()->updateCurrentZoomResource();
 
         d->viewConnections.addUniqueConnection(
                     image(), SIGNAL(sigSizeChanged(QPointF,QPointF)),
@@ -910,7 +910,8 @@ void KisViewManager::slotSaveIncremental()
     QString version = "000";
     QString newVersion;
     QString letter;
-    QString fileName = document()->localFilePath();
+    QString path = QFileInfo(document()->localFilePath()).canonicalPath();
+    QString fileName = QFileInfo(document()->localFilePath()).fileName();
 
     // Find current version filenames
     // v v Regexp to find incremental versions in the filename, taking our backup scheme into account as well
@@ -970,7 +971,7 @@ void KisViewManager::slotSaveIncremental()
             newVersion.append(".");
         }
         fileName.replace(regex, newVersion);
-        fileAlreadyExists = QFile(fileName).exists();
+        fileAlreadyExists = QFileInfo(path + '/' + fileName).exists();
         if (fileAlreadyExists) {
             if (!letter.isNull()) {
                 char letterCh = letter.at(0).toLatin1();
@@ -986,9 +987,11 @@ void KisViewManager::slotSaveIncremental()
         QMessageBox::critical(mainWindow(), i18nc("@title:window", "Couldn't save incremental version"), i18n("Alternative names exhausted, try manually saving with a higher number"));
         return;
     }
+    QUrl newUrl = QUrl::fromUserInput(path + '/' + fileName);
     document()->setFileBatchMode(true);
-    document()->saveAs(QUrl::fromUserInput(fileName), document()->mimeType(), true);
+    document()->saveAs(newUrl, document()->mimeType(), true);
     document()->setFileBatchMode(false);
+    KisPart::instance()->addRecentURLToAllMainWindows(newUrl, document()->url());
 
     if (mainWindow()) {
         mainWindow()->updateCaption();
@@ -1011,7 +1014,8 @@ void KisViewManager::slotSaveIncrementalBackup()
     QString version = "000";
     QString newVersion;
     QString letter;
-    QString fileName = document()->localFilePath();
+    QString path = QFileInfo(document()->localFilePath()).canonicalPath();
+    QString fileName = QFileInfo(document()->localFilePath()).fileName();
 
     // First, discover if working on a backup file, or a normal file
     QRegExp regex("~\\d{1,4}[.]|~\\d{1,4}[a-z][.]");
@@ -1047,7 +1051,7 @@ void KisViewManager::slotSaveIncrementalBackup()
             if (!letter.isNull()) newVersion.append(letter);
             newVersion.append(".");
             backupFileName.replace(regex, newVersion);
-            fileAlreadyExists = QFile(backupFileName).exists();
+            fileAlreadyExists = QFile(path + '/' + backupFileName).exists();
             if (fileAlreadyExists) {
                 if (!letter.isNull()) {
                     char letterCh = letter.at(0).toLatin1();
@@ -1063,8 +1067,8 @@ void KisViewManager::slotSaveIncrementalBackup()
             QMessageBox::critical(mainWindow(), i18nc("@title:window", "Couldn't save incremental backup"), i18n("Alternative names exhausted, try manually saving with a higher number"));
             return;
         }
-        QFile::copy(fileName, backupFileName);
-        document()->saveAs(QUrl::fromUserInput(fileName), document()->mimeType(), true);
+        QFile::copy(path + '/' + fileName, path + '/' + backupFileName);
+        document()->saveAs(QUrl::fromUserInput(path + '/' + fileName), document()->mimeType(), true);
 
         if (mainWindow()) mainWindow()->updateCaption();
     }
@@ -1072,7 +1076,7 @@ void KisViewManager::slotSaveIncrementalBackup()
         // Navigate directory searching for latest backup version, ignore letters
         const quint8 HARDCODED_DIGIT_COUNT = 3;
         QString baseNewVersion = "000";
-        QString backupFileName = document()->localFilePath();
+        QString backupFileName = QFileInfo(document()->localFilePath()).fileName();
         QRegExp regex2("[.][a-z]{2,4}$");  //  Heuristic to find file extension
         regex2.indexIn(backupFileName);
         QStringList matches2 = regex2.capturedTexts();
@@ -1087,7 +1091,7 @@ void KisViewManager::slotSaveIncrementalBackup()
             newVersion.prepend("~");
             newVersion.append(".");
             backupFileName.replace(regex, newVersion);
-            fileAlreadyExists = QFile(backupFileName).exists();
+            fileAlreadyExists = QFile(path + '/' + backupFileName).exists();
             if (fileAlreadyExists) {
                 // Prepare the base for new version filename, increment by 1
                 int intVersion = baseNewVersion.toInt(0);
@@ -1101,8 +1105,8 @@ void KisViewManager::slotSaveIncrementalBackup()
 
         // Save both as backup and on current file for interapplication workflow
         document()->setFileBatchMode(true);
-        QFile::copy(fileName, backupFileName);
-        document()->saveAs(QUrl::fromUserInput(fileName), document()->mimeType(), true);
+        QFile::copy(path + '/' + fileName, path + '/' + backupFileName);
+        document()->saveAs(QUrl::fromUserInput(path + '/' + fileName), document()->mimeType(), true);
         document()->setFileBatchMode(false);
 
         if (mainWindow()) mainWindow()->updateCaption();
@@ -1244,7 +1248,7 @@ void KisViewManager::switchCanvasOnly(bool toggled)
         // show a fading heads-up display about the shortcut to go back
 
         showFloatingMessage(i18n("Going into Canvas-Only mode.\nPress %1 to go back.",
-                                 actionCollection()->action("view_show_canvas_only")->shortcut().toString()), QIcon());
+                                 actionCollection()->action("view_show_canvas_only")->shortcut().toString(QKeySequence::NativeText)), QIcon());
     }
     else {
         main->restoreState(d->canvasState);
@@ -1290,7 +1294,7 @@ void KisViewManager::guiUpdateTimeout()
     d->selectionManager.updateGUI();
     d->filterManager.updateGUI();
     if (zoomManager()) {
-        zoomManager()->updateGUI();
+        zoomManager()->updateGuiAfterDocumentSize();
     }
     d->gridManager.updateGUI();
     d->actionManager.updateGUI();
@@ -1440,4 +1444,16 @@ void KisViewManager::slotResetFgBg()
     d->canvasResourceManager.setForegroundColor(KoColor(Qt::black, KoColorSpaceRegistry::instance()->rgb8()));
 }
 
+void KisViewManager::slotResetRotation()
+{
+    KisCanvasController *canvasController = d->currentImageView->canvasController();
+    canvasController->resetCanvasRotation();
+}
 
+void KisViewManager::slotToggleFullscreen()
+{
+    KisConfig cfg(false);
+    KisMainWindow *main = mainWindow();
+    main->viewFullscreen(!main->isFullScreen());
+    cfg.fullscreenMode(main->isFullScreen());
+}
