@@ -52,11 +52,12 @@ public:
     void end(QEvent *event) override;
 
     void inputEvent(QEvent* event) override;
-    void cursorMoved(const QPointF &lastPos, const QPointF &pos) override;
+    void cursorMovedAbsolute(const QPointF &lastPos, const QPointF &pos) override;
 
     bool isShortcutRequired(int shortcut) const override;
 
     KisInputActionGroup inputActionGroup(int shortcut) const override;
+    bool supportsHiResInputEvents(int shortcut) const override;
 
 private:
     class Private;
