@@ -26,7 +26,7 @@ KisGamutMaskToolbar::KisGamutMaskToolbar(QWidget* parent) : QWidget(parent)
   , m_selectedMask(nullptr)
   , m_selfUpdate(false)
 {
-    m_ui = new Ui_wdgGamutMaskToolbar();
+    m_ui.reset(new Ui_wdgGamutMaskToolbar());
     m_ui->setupUi(this);
 
     m_iconMaskOff = KisIconUtils::loadIcon("gamut-mask-off");
@@ -38,15 +38,13 @@ KisGamutMaskToolbar::KisGamutMaskToolbar(QWidget* parent) : QWidget(parent)
     m_ui->bnToggleMask->setChecked(false);
     m_ui->bnToggleMask->setIcon(m_iconMaskOff);
 
-    m_ui->rotationSlider->setRange(0, 360);
-    m_ui->rotationSlider->setPrefix(i18n("Rotation: "));
-    m_ui->rotationSlider->setSuffix("°");
-    m_ui->rotationSlider->setFastSliderStep(30); // TODO: test for usability
-    m_ui->rotationSlider->hide();
+    m_ui->rotationAngleSelector->setDecimals(0);
+    m_ui->rotationAngleSelector->setIncreasingDirection(KisAngleGauge::IncreasingDirection_Clockwise);
+    m_ui->rotationAngleSelector->hide();
 
     // gamut mask connections
     connect(m_ui->bnToggleMask, SIGNAL(toggled(bool)), SLOT(slotGamutMaskToggle(bool)));
-    connect(m_ui->rotationSlider, SIGNAL(valueChanged(int)), SLOT(slotGamutMaskRotate(int)));
+    connect(m_ui->rotationAngleSelector, SIGNAL(angleChanged(qreal)), SLOT(slotGamutMaskRotate(qreal)));
 }
 
 void KisGamutMaskToolbar::connectMaskSignals(KisCanvasResourceProvider* resourceProvider)
@@ -88,7 +86,7 @@ void KisGamutMaskToolbar::slotGamutMaskSet(KoGamutMask *mask)
 
 void KisGamutMaskToolbar::slotGamutMaskUnset()
 {
-    m_ui->rotationSlider->hide();
+    m_ui->rotationAngleSelector->hide();
     m_ui->labelMaskName->show();
     m_ui->labelMaskName->setText(m_textNoMask);
 }
@@ -111,11 +109,11 @@ void KisGamutMaskToolbar::slotGamutMaskToggle(bool state)
     if (b == true) {
         m_ui->bnToggleMask->setIcon(m_iconMaskOn);
         m_ui->labelMaskName->hide();
-        m_ui->rotationSlider->show();
+        m_ui->rotationAngleSelector->show();
 
-        m_ui->rotationSlider->blockSignals(true);
-        m_ui->rotationSlider->setValue(m_selectedMask->rotation());
-        m_ui->rotationSlider->blockSignals(false);
+        m_ui->rotationAngleSelector->blockSignals(true);
+        m_ui->rotationAngleSelector->setAngle(static_cast<qreal>(m_selectedMask->rotation()));
+        m_ui->rotationAngleSelector->blockSignals(false);
 
         m_selfUpdate = true;
         emit sigGamutMaskChanged(m_selectedMask);
@@ -123,7 +121,7 @@ void KisGamutMaskToolbar::slotGamutMaskToggle(bool state)
 
     } else {
         m_ui->bnToggleMask->setIcon(m_iconMaskOff);
-        m_ui->rotationSlider->hide();
+        m_ui->rotationAngleSelector->hide();
         m_ui->labelMaskName->show();
         m_ui->labelMaskName->setText(m_textMaskDisabled);
 
@@ -133,12 +131,12 @@ void KisGamutMaskToolbar::slotGamutMaskToggle(bool state)
     }
 }
 
-void KisGamutMaskToolbar::slotGamutMaskRotate(int angle)
+void KisGamutMaskToolbar::slotGamutMaskRotate(qreal angle)
 {
     if (!m_selectedMask) {
         return;
     }
 
-    m_selectedMask->setRotation(angle);
+    m_selectedMask->setRotation(static_cast<int>(angle));
     emit sigGamutMaskChanged(m_selectedMask);
 }
