@@ -32,9 +32,12 @@
 #include <resources/KoPattern.h>
 #include <resources/KoAbstractGradient.h>
 #include <resources/KoColorSet.h>
-#include <resources/KoAbstractGradient.h>
 #include <resources/KoSvgSymbolCollectionResource.h>
-
+#include <resources/KoGamutMask.h>
+#include <config-seexpr.h>
+#if defined HAVE_SEEXPR
+#include <resources/KisSeExprScript.h>
+#endif
 
 /**
  * Provides default resource servers for gradients, patterns and palettes
@@ -62,6 +65,10 @@ public:
     KoResourceServer<KoAbstractGradient>* gradientServer();
     KoResourceServer<KoColorSet>* paletteServer();
     KoResourceServer<KoSvgSymbolCollectionResource>* svgSymbolCollectionServer();
+    KoResourceServer<KoGamutMask>* gamutMaskServer();
+#if defined HAVE_SEEXPR
+    KoResourceServer<KisSeExprScript> *seExprScriptServer();
+#endif
 
 private:
     KoResourceServerProvider(const KoResourceServerProvider&);
