@@ -3,7 +3,8 @@
  *
  *  This library is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; version 2.1 of the License.
+ *  the Free Software Foundation; version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,7 +34,8 @@
 
 #include <kis_debug.h>
 
-#include <KoCanvasResourceManager.h>
+#include <KoCanvasResourceProvider.h>
+#include <kis_canvas_resource_provider.h>
 #include <kis_icon.h>
 
 #include "kis_color_selector_ring.h"
@@ -128,14 +130,18 @@ void KisColorSelector::setConfiguration(KisColorSelectorConfiguration conf)
     connect(m_mainComponent, SIGNAL(paramChanged(qreal,qreal,qreal,qreal,qreal,qreal,qreal,qreal,qreal)),
             m_subComponent,  SLOT(setParam(qreal,qreal,qreal,qreal,qreal,qreal,qreal,qreal,qreal)), Qt::UniqueConnection);
     connect(m_subComponent,  SIGNAL(paramChanged(qreal,qreal,qreal,qreal,qreal,qreal,qreal,qreal,qreal)),
-            m_mainComponent, SLOT(setParam(qreal,qreal,qreal,qreal, qreal, qreal, qreal, qreal, qreal)), Qt::UniqueConnection);
+            m_mainComponent, SLOT(setParam(qreal,qreal,qreal,qreal,qreal,qreal,qreal,qreal,qreal)), Qt::UniqueConnection);
 
     connect(m_mainComponent, SIGNAL(update()), m_signalCompressor, SLOT(start()), Qt::UniqueConnection);
     connect(m_subComponent,  SIGNAL(update()), m_signalCompressor, SLOT(start()), Qt::UniqueConnection);
 
     m_mainComponent->setConfiguration(m_configuration.mainTypeParameter, m_configuration.mainType);
-    m_subComponent->setConfiguration(m_configuration.subTypeParameter, m_configuration.subType);
 
+    // Use Hluma as subType when the main type uses hsy'.
+    KisColorSelectorConfiguration::Parameters subTypeParameter = m_configuration.mainTypeParameter == KisColorSelectorConfiguration::SY ?
+            KisColorSelectorConfiguration::Hluma : m_configuration.subTypeParameter;
+
+    m_subComponent->setConfiguration(subTypeParameter, m_configuration.subType);
     QResizeEvent event(QSize(width(), height()), QSize());
     resizeEvent(&event);
 }
@@ -149,7 +155,54 @@ void KisColorSelector::updateSettings()
 {
     KisColorSelectorBase::updateSettings();
     KConfigGroup cfg =  KSharedConfig::openConfig()->group("advancedColorSelector");
+
     setConfiguration(KisColorSelectorConfiguration::fromString(cfg.readEntry("colorSelectorConfiguration", KisColorSelectorConfiguration().toString())));
+
+    if (m_canvas && m_canvas->viewManager() && m_canvas->viewManager()->canvasResourceProvider()) {
+        bool gamutMaskActive = m_canvas->viewManager()->canvasResourceProvider()->gamutMaskActive();
+
+        if (gamutMaskActive) {
+            KoGamutMask* currentMask = m_canvas->viewManager()->canvasResourceProvider()->currentGamutMask();
+            if (currentMask) {
+                slotGamutMaskSet(currentMask);
+            }
+        } else {
+            slotGamutMaskToggle(false);
+        }
+    }
+}
+
+void KisColorSelector::slotGamutMaskSet(KoGamutMask *gamutMask)
+{
+    m_mainComponent->setGamutMask(gamutMask);
+    m_subComponent->setGamutMask(gamutMask);
+
+    slotGamutMaskToggle(true);
+}
+
+void KisColorSelector::slotGamutMaskUnset()
+{
+    m_mainComponent->unsetGamutMask();
+    m_subComponent->unsetGamutMask();
+
+    slotGamutMaskToggle(false);
+}
+
+void KisColorSelector::slotGamutMaskPreviewUpdate()
+{
+    m_mainComponent->updateGamutMaskPreview();
+    m_subComponent->updateGamutMaskPreview();
+}
+
+void KisColorSelector::slotGamutMaskDeactivate()
+{
+    slotGamutMaskToggle(false);
+}
+
+void KisColorSelector::slotGamutMaskToggle(bool state)
+{
+    m_mainComponent->toggleGamutMask(state);
+    m_subComponent->toggleGamutMask(state);
 }
 
 void KisColorSelector::updateIcons() {
@@ -165,8 +218,6 @@ void KisColorSelector::hasAtLeastOneDocument(bool value)
 
 void KisColorSelector::reset()
 {
-    KisColorSelectorBase::reset();
-
     if (m_mainComponent) {
         m_mainComponent->setDirty();
     }
@@ -174,6 +225,8 @@ void KisColorSelector::reset()
     if (m_subComponent) {
         m_subComponent->setDirty();
     }
+
+    KisColorSelectorBase::reset();
 }
 
 void KisColorSelector::paintEvent(QPaintEvent* e)
