@@ -22,6 +22,7 @@ typedef unsigned int QRgb;
 
 #include <QColor>
 #include <QImage>
+#include <QSize>
 
 #include "KoColor.h"
 #include "kis_color_selector_component.h"
@@ -43,16 +44,22 @@ protected:
 
 private:
     friend class Acs::PixelCacheRenderer;
-    KoColor colorAt(int x, int y, bool forceValid = false);
+    KoColor colorAt(float x, float y, bool forceValid = false);
 
 private:
+    bool allowsColorSelectionAtPoint(const QPoint &pt) const override;
     QPointF m_lastClickPos;
     QImage m_pixelCache;
     QPoint m_pixelCacheOffset;
-    qreal R;
-    qreal G;
-    qreal B;
-    qreal Gamma;
+    qreal R {0.0};
+    qreal G {0.0};
+    qreal B {0.0};
+    qreal Gamma {0.0};
+
+    QSize m_renderAreaSize;
+    qreal m_renderAreaOffsetX {0.0};
+    qreal m_renderAreaOffsetY {0.0};
+    QTransform m_toRenderArea;
 };
 
 #endif // KIS_COLOR_SELECTOR_WHEEL_H
