@@ -49,6 +49,11 @@ DlgClonesArray::DlgClonesArray(KisViewManager *viewManager, QWidget *parent)
     Q_CHECK_PTR(m_page);
     m_page->setObjectName("clones_array");
 
+    m_page->columnAngle->setRange(-360, 360);
+    m_page->columnAngle->setFlipOptionsMode(KisAngleSelector::FlipOptionsMode_NoFlipOptions);
+    m_page->rowAngle->setRange(-360, 360);
+    m_page->rowAngle->setFlipOptionsMode(KisAngleSelector::FlipOptionsMode_NoFlipOptions);
+
     setMainWidget(m_page);
     resize(m_page->sizeHint());
 
@@ -62,9 +67,9 @@ DlgClonesArray::DlgClonesArray(KisViewManager *viewManager, QWidget *parent)
     connect(m_page->rowYOffset, SIGNAL(valueChanged(int)), SLOT(syncOrthogonalToAngular()));
 
     connect(m_page->columnDistance, SIGNAL(valueChanged(double)), SLOT(syncAngularToOrthogonal()));
-    connect(m_page->columnAngle, SIGNAL(valueChanged(double)), SLOT(syncAngularToOrthogonal()));
+    connect(m_page->columnAngle, SIGNAL(angleChanged(qreal)), SLOT(syncAngularToOrthogonal()));
     connect(m_page->rowDistance, SIGNAL(valueChanged(double)), SLOT(syncAngularToOrthogonal()));
-    connect(m_page->rowAngle, SIGNAL(valueChanged(double)), SLOT(syncAngularToOrthogonal()));
+    connect(m_page->rowAngle, SIGNAL(angleChanged(qreal)), SLOT(syncAngularToOrthogonal()));
 
     connect(m_page->numNegativeColumns, SIGNAL(valueChanged(int)), SLOT(setDirty()));
     connect(m_page->numPositiveColumns, SIGNAL(valueChanged(int)), SLOT(setDirty()));
@@ -124,12 +129,12 @@ void DlgClonesArray::syncOrthogonalToAngular()
     x = m_page->columnXOffset->value();
     y = m_page->columnYOffset->value();
     m_page->columnDistance->setValue((float)sqrt(pow2(x) + pow2(y)));
-    m_page->columnAngle->setValue(kisRadiansToDegrees(atan2((double) y, (double) x)));
+    m_page->columnAngle->setAngle(kisRadiansToDegrees(atan2((double) y, (double) x)));
 
     x = m_page->rowXOffset->value();
     y = m_page->rowYOffset->value();
     m_page->rowDistance->setValue((float)sqrt(pow2(x) + pow2(y)));
-    m_page->rowAngle->setValue(kisRadiansToDegrees(atan2((double) y, (double) x)));
+    m_page->rowAngle->setAngle(kisRadiansToDegrees(atan2((double) y, (double) x)));
 
     setAngularSignalsEnabled(true);
     setDirty();
@@ -142,12 +147,12 @@ void DlgClonesArray::syncAngularToOrthogonal()
     qreal a, d;
 
     d = m_page->columnDistance->value();
-    a = kisDegreesToRadians(m_page->columnAngle->value());
+    a = kisDegreesToRadians(m_page->columnAngle->angle());
     m_page->columnXOffset->setValue(qRound(d * cos(a)));
     m_page->columnYOffset->setValue(qRound(d * sin(a)));
 
     d = m_page->rowDistance->value();
-    a = kisDegreesToRadians(m_page->rowAngle->value());
+    a = kisDegreesToRadians(m_page->rowAngle->angle());
     m_page->rowXOffset->setValue(qRound(d * cos(a)));
     m_page->rowYOffset->setValue(qRound(d * sin(a)));
 
@@ -223,14 +228,14 @@ void DlgClonesArray::reapplyClones()
     int endColumn = m_page->numPositiveColumns->value() - 1;
     int endRow = m_page->numPositiveRows->value() - 1;
 
-    QString positiveGroupName = QString(i18n("+ Array of %1")).arg(m_baseLayer->name());
+    QString positiveGroupName = i18n("+ Array of %1", m_baseLayer->name());
     KisGroupLayerSP positiveGroupLayer = new KisGroupLayer(image, positiveGroupName, OPACITY_OPAQUE_U8);
     m_applicator->applyCommand(new KisImageLayerAddCommand(image, positiveGroupLayer, m_baseLayer->parent(), m_baseLayer, false, true), KisStrokeJobData::SEQUENTIAL, KisStrokeJobData::EXCLUSIVE);
 
     KisGroupLayerSP negativeGroupLayer;
 
     if (startRow < 0 || startColumn < 0) {
-        QString negativeGroupName = QString(i18n("- Array of %1")).arg(m_baseLayer->name());
+        QString negativeGroupName = i18n("- Array of %1", m_baseLayer->name());
         negativeGroupLayer = new KisGroupLayer(image, negativeGroupName, OPACITY_OPAQUE_U8);
         m_applicator->applyCommand(new KisImageLayerAddCommand(image, negativeGroupLayer, m_baseLayer->parent(), m_baseLayer->prevSibling(), false, true), KisStrokeJobData::SEQUENTIAL, KisStrokeJobData::EXCLUSIVE);
     }
@@ -243,7 +248,7 @@ void DlgClonesArray::reapplyClones()
             KisNodeSP parent = choosePositiveGroup ? positiveGroupLayer : negativeGroupLayer;
 
 
-            QString cloneName = QString("Clone %1, %2").arg(col).arg(row);
+            QString cloneName = i18n("Clone %1, %2", col, row);
             KisCloneLayerSP clone = new KisCloneLayer(m_baseLayer, image, cloneName, OPACITY_OPAQUE_U8);
             clone->setX(-row * rowXOffset + col * columnXOffset);
             clone->setY(-row * rowYOffset + col * columnYOffset);
