@@ -32,7 +32,7 @@ KritaPyQtPlugin::KritaPyQtPlugin(QObject *parent, const QVariantList &)
     : QObject(parent)
     , m_autoReload(false)
 {
-    qDebug() << "Loading Python plugin";
+    dbgScript << "Loading Python plugin";
 
     PyKrita::InitResult initResult = PyKrita::initialize();
     switch (initResult) {
@@ -89,7 +89,10 @@ KritaPyQtPlugin::KritaPyQtPlugin(QObject *parent, const QVariantList &)
 
 KritaPyQtPlugin::~KritaPyQtPlugin()
 {
-    PyKrita::finalize();
+    /// XXX: Don't call PyKrita::finalize here, because that can result in a crash
+    ///      deep inside Qt. See https://bugs.kde.org/show_bug.cgi?id=417465
+
+    // PyKrita::finalize();
 }
 
 #include "plugin.moc"
