@@ -22,10 +22,13 @@
 #include "kis_kra_save_visitor.h"
 #include "kis_kra_savexml_visitor.h"
 
+#include <QApplication>
+#include <QMessageBox>
 #include <QDomDocument>
 #include <QDomElement>
 #include <QString>
 #include <QStringList>
+#include <QScopedPointer>
 
 #include <QUrl>
 #include <QBuffer>
@@ -35,12 +38,16 @@
 #include <KoColorSpace.h>
 #include <KoColorProfile.h>
 #include <KoColor.h>
+#include <KoColorSet.h>
+#include <KoResourceServer.h>
+#include <KoResourceServerProvider.h>
 #include <KoStore.h>
 #include <KoStoreDevice.h>
 
 #include <kis_annotation.h>
 #include <kis_image.h>
 #include <kis_image_animation_interface.h>
+#include <KisImportExportManager.h>
 #include <kis_group_layer.h>
 #include <kis_layer.h>
 #include <kis_adjustment_layer.h>
@@ -57,6 +64,8 @@
 #include "kis_guides_config.h"
 #include "KisProofingConfiguration.h"
 
+#include <KisMirrorAxisConfig.h>
+
 #include <QFileInfo>
 #include <QDir>
 
@@ -70,13 +79,15 @@ public:
     QMap<const KisNode*, QString> nodeFileNames;
     QMap<const KisNode*, QString> keyframeFilenames;
     QString imageName;
+    QString filename;
     QStringList errorMessages;
 };
 
-KisKraSaver::KisKraSaver(KisDocument* document)
+KisKraSaver::KisKraSaver(KisDocument* document, const QString &filename)
         : m_d(new Private)
 {
     m_d->doc = document;
+    m_d->filename = filename;
 
     m_d->imageName = m_d->doc->documentInfo()->aboutInfo("title");
     if (m_d->imageName.isEmpty()) {
@@ -108,11 +119,13 @@ QDomElement KisKraSaver::saveXML(QDomDocument& doc,  KisImageSP image)
     imageElement.setAttribute(Y_RESOLUTION, KisDomUtils::toString(image->yRes()*72.0));
     //now the proofing options:
     if (image->proofingConfiguration()) {
-        imageElement.setAttribute(PROOFINGPROFILENAME, KisDomUtils::toString(image->proofingConfiguration()->proofingProfile));
-        imageElement.setAttribute(PROOFINGMODEL, KisDomUtils::toString(image->proofingConfiguration()->proofingModel));
-        imageElement.setAttribute(PROOFINGDEPTH, KisDomUtils::toString(image->proofingConfiguration()->proofingDepth));
-        imageElement.setAttribute(PROOFINGINTENT, KisDomUtils::toString(image->proofingConfiguration()->intent));
-        imageElement.setAttribute(PROOFINGADAPTATIONSTATE, KisDomUtils::toString(image->proofingConfiguration()->adaptationState));
+        if (image->proofingConfiguration()->storeSoftproofingInsideImage) {
+            imageElement.setAttribute(PROOFINGPROFILENAME, KisDomUtils::toString(image->proofingConfiguration()->proofingProfile));
+            imageElement.setAttribute(PROOFINGMODEL, KisDomUtils::toString(image->proofingConfiguration()->proofingModel));
+            imageElement.setAttribute(PROOFINGDEPTH, KisDomUtils::toString(image->proofingConfiguration()->proofingDepth));
+            imageElement.setAttribute(PROOFINGINTENT, KisDomUtils::toString(image->proofingConfiguration()->intent));
+            imageElement.setAttribute(PROOFINGADAPTATIONSTATE, KisDomUtils::toString(image->proofingConfiguration()->adaptationState));
+        }
     }
 
     quint32 count = 1; // We don't save the root layer, but it does count
@@ -130,9 +143,11 @@ QDomElement KisKraSaver::saveXML(QDomDocument& doc,  KisImageSP image)
     saveWarningColor(doc, imageElement, image);
     saveCompositions(doc, imageElement, image);
     saveAssistantsList(doc, imageElement);
-    saveGrid(doc,imageElement);
-    saveGuides(doc,imageElement);
-    saveAudio(doc,imageElement);
+    saveGrid(doc, imageElement);
+    saveGuides(doc, imageElement);
+    saveMirrorAxis(doc, imageElement);
+    saveAudio(doc, imageElement);
+    savePalettesToXML(doc, imageElement);
 
     QDomElement animationElement = doc.createElement("animation");
     KisDomUtils::saveValue(&animationElement, "framerate", image->animationInterface()->framerate());
@@ -143,6 +158,54 @@ QDomElement KisKraSaver::saveXML(QDomDocument& doc,  KisImageSP image)
     return imageElement;
 }
 
+bool KisKraSaver::savePalettes(KoStore *store, KisImageSP image, const QString &uri)
+{
+    Q_UNUSED(image);
+    Q_UNUSED(uri);
+
+    bool res = true;
+    if (m_d->doc->paletteList().size() == 0) {
+        return true;
+    }
+    for (const KoColorSet *palette : m_d->doc->paletteList()) {
+        if (!palette->isGlobal()) {
+            if (!store->open(m_d->imageName + PALETTE_PATH + palette->filename())) {
+                m_d->errorMessages << i18nc("Error message when saving a .kra file", "Could not open store for saving the document palette.");
+                return false;
+            }
+            QByteArray ba = palette->toByteArray();
+            qint64 nwritten = 0;
+            if (!ba.isEmpty()) {
+                nwritten = store->write(ba);
+            } else {
+                qWarning() << "Cannot save the palette to a byte array because it is empty:" << palette->name();
+            }
+            res = store->close();
+
+            res = res && (nwritten == ba.size());
+        }
+    }
+    if (!res) {
+        m_d->errorMessages << i18nc("Error message when saving a .kra file", "Could not save palettes.");
+        qWarning() << "Could not save palette";
+    }
+    return res;
+}
+
+
+void KisKraSaver::savePalettesToXML(QDomDocument &doc, QDomElement &element)
+{
+    QDomElement ePalette = doc.createElement(PALETTES);
+    for (const KoColorSet *palette : m_d->doc->paletteList()) {
+        if (!palette->isGlobal()) {
+            QDomElement eFile =  doc.createElement("palette");
+            eFile.setAttribute("filename", palette->filename());
+            ePalette.appendChild(eFile);
+        }
+    }
+    element.appendChild(ePalette);
+}
+
 bool KisKraSaver::saveKeyframes(KoStore *store, const QString &uri, bool external)
 {
     QMap<const KisNode*, QString>::iterator it;
@@ -174,12 +237,17 @@ bool KisKraSaver::saveNodeKeyframes(KoStore *store, QString location, const KisN
         root.appendChild(element);
     }
 
+    bool success = true;
     if (store->open(location)) {
         QByteArray xml = doc.toByteArray();
-        store->write(xml);
-        store->close();
+        qint64 nwritten = store->write(xml);
+        bool r = store->close();
+        success = r && (nwritten == xml.size());
     } else {
-        m_d->errorMessages << i18n("could not save keyframes");
+        success = false;
+    }
+    if (!success) {
+        m_d->errorMessages << i18nc("Error message on saving a .kra file", "Could not save keyframes.");
         return false;
     }
 
@@ -203,19 +271,35 @@ bool KisKraSaver::saveBinaryData(KoStore* store, KisImageSP image, const QString
         return false;
     }
 
+    bool success = true;
+    bool r = true;
+    qint64 nwritten = 0;
+
     // saving annotations
     // XXX this only saves EXIF and ICC info. This would probably need
     // a redesign of the dtd of the krita file to do this more generally correct
     // e.g. have <ANNOTATION> tags or so.
+    bool savingAnnotationsSuccess = true;
     KisAnnotationSP annotation = image->annotation("exif");
     if (annotation) {
         location = external ? QString() : uri;
         location += m_d->imageName + EXIF_PATH;
         if (store->open(location)) {
-            store->write(annotation->annotation());
-            store->close();
+            nwritten = store->write(annotation->annotation());
+            r = store->close();
+            savingAnnotationsSuccess = savingAnnotationsSuccess && (nwritten == annotation->annotation().size()) && r;
+        } else {
+            savingAnnotationsSuccess = false;
         }
     }
+
+    if (!savingAnnotationsSuccess) {
+        m_d->errorMessages.append(i18nc("Saving .kra file error message", "Could not save annotations."));
+    }
+
+    success = success && savingAnnotationsSuccess;
+
+    bool savingImageProfileSuccess = true;
     if (image->profile()) {
         const KoColorProfile *profile = image->profile();
         KisAnnotationSP annotation;
@@ -234,32 +318,52 @@ bool KisKraSaver::saveBinaryData(KoStore* store, KisImageSP image, const QString
             location = external ? QString() : uri;
             location += m_d->imageName + ICC_PATH;
             if (store->open(location)) {
-                store->write(annotation->annotation());
-                store->close();
+                nwritten = store->write(annotation->annotation());
+                r = store->close();
+                savingImageProfileSuccess = savingImageProfileSuccess && (nwritten == annotation->annotation().size()) && r;
+            } else {
+                savingImageProfileSuccess = false;
             }
         }
     }
 
+    if (!savingImageProfileSuccess) {
+        m_d->errorMessages.append(i18nc("Saving .kra file error message", "Could not save image profile."));
+    }
+    success = success && savingImageProfileSuccess;
+
     //This'll embed the profile used for proofing into the kra file.
+    bool savingSoftproofingProfileSuccess = true;
     if (image->proofingConfiguration()) {
-        const KoColorProfile *proofingProfile = KoColorSpaceRegistry::instance()->profileByName(image->proofingConfiguration()->proofingProfile);
-        if (proofingProfile && proofingProfile->valid()) {
-            QByteArray proofingProfileRaw = proofingProfile->rawData();
-            if (!proofingProfileRaw.isEmpty()) {
-                annotation = new KisAnnotation(ICCPROOFINGPROFILE, proofingProfile->name(), proofingProfile->rawData());
+        if (image->proofingConfiguration()->storeSoftproofingInsideImage) {
+            const KoColorProfile *proofingProfile = KoColorSpaceRegistry::instance()->profileByName(image->proofingConfiguration()->proofingProfile);
+            if (proofingProfile && proofingProfile->valid()) {
+                QByteArray proofingProfileRaw = proofingProfile->rawData();
+                if (!proofingProfileRaw.isEmpty()) {
+                    annotation = new KisAnnotation(ICCPROOFINGPROFILE, proofingProfile->name(), proofingProfile->rawData());
+                }
+            }
+            if (annotation) {
+                location = external ? QString() : uri;
+                location += m_d->imageName + ICC_PROOFING_PATH;
+                if (store->open(location)) {
+                    nwritten = store->write(annotation->annotation());
+                    r = store->close();
+                    savingSoftproofingProfileSuccess = savingSoftproofingProfileSuccess && (nwritten == annotation->annotation().size()) && r;
+                } else {
+                    savingSoftproofingProfileSuccess = false;
+                }
             }
         }
-        if (annotation) {
-            location = external ? QString() : uri;
-            location += m_d->imageName + ICC_PROOFING_PATH;
-            if (store->open(location)) {
-                store->write(annotation->annotation());
-                store->close();
-            }
-        }
-
     }
 
+    if (!savingSoftproofingProfileSuccess) {
+        m_d->errorMessages.append(i18nc("Saving .kra file error message", "Could not save softproofing color profile."));
+    }
+
+    success = success && savingSoftproofingProfileSuccess;
+
+    bool savingLayerStylesSuccess = true;
     {
         KisPSDLayerStyleCollectionResource collection("not-nexists.asl");
         KIS_ASSERT_RECOVER_NOOP(!collection.valid());
@@ -270,23 +374,49 @@ bool KisKraSaver::saveBinaryData(KoStore* store, KisImageSP image, const QString
 
             if (store->open(location)) {
                 QBuffer aslBuffer;
-                aslBuffer.open(QIODevice::WriteOnly);
-                collection.saveToDevice(&aslBuffer);
-                aslBuffer.close();
-
-                store->write(aslBuffer.buffer());
-                store->close();
+                if (aslBuffer.open(QIODevice::WriteOnly)) {
+                    collection.saveToDevice(&aslBuffer);
+                    aslBuffer.close();
+                    nwritten = store->write(aslBuffer.buffer());
+                    savingLayerStylesSuccess = savingLayerStylesSuccess && (nwritten == aslBuffer.buffer().size());
+                } else {
+                    savingLayerStylesSuccess = false;
+                }
+                r = store->close();
+                savingLayerStylesSuccess = savingLayerStylesSuccess && r;
+            } else {
+                savingLayerStylesSuccess = false;
             }
         }
     }
 
-    if (!autosave) {
-        KisPaintDeviceSP dev = image->projection();
-        KisPNGConverter::saveDeviceToStore("mergedimage.png", image->bounds(), image->xRes(), image->yRes(), dev, store);
+    if (!savingLayerStylesSuccess) {
+        m_d->errorMessages.append(i18nc("Saving .kra file error message", "Could not save layer styles."));
     }
 
-    saveAssistants(store, uri,external);
-    return true;
+    success = success && savingLayerStylesSuccess;
+
+    bool savingMergedImageSuccess = true;
+
+
+    if (!autosave) {
+        KisPaintDeviceSP dev = image->projection();
+        store->setCompressionEnabled(false);
+        r = KisPNGConverter::saveDeviceToStore("mergedimage.png", image->bounds(), image->xRes(), image->yRes(), dev, store);
+        savingMergedImageSuccess = savingMergedImageSuccess && r;
+        store->setCompressionEnabled(KisConfig(true).compressKra());
+    }
+
+    if (!savingMergedImageSuccess) {
+        m_d->errorMessages.append(i18nc("Saving .kra file error message", "Could not save merged image."));
+    }
+
+    success = success && savingMergedImageSuccess;
+
+    r = saveAssistants(store, uri,external);
+    success = success && r;
+
+    return success;
 }
 
 QStringList KisKraSaver::errorMessages() const
@@ -314,10 +444,12 @@ void KisKraSaver::saveAssistantsGlobalColor(QDomDocument& doc, QDomElement& elem
 void KisKraSaver::saveWarningColor(QDomDocument& doc, QDomElement& element, KisImageSP image)
 {
     if (image->proofingConfiguration()) {
-        QDomElement e = doc.createElement(PROOFINGWARNINGCOLOR);
-        KoColor color = image->proofingConfiguration()->warningColor;
-        color.toXML(doc, e);
-        element.appendChild(e);
+        if (image->proofingConfiguration()->storeSoftproofingInsideImage) {
+            QDomElement e = doc.createElement(PROOFINGWARNINGCOLOR);
+            KoColor color = image->proofingConfiguration()->warningColor;
+            color.toXML(doc, e);
+            element.appendChild(e);
+        }
     }
 }
 
@@ -340,6 +472,8 @@ bool KisKraSaver::saveAssistants(KoStore* store, QString uri, bool external)
 
     QList<KisPaintingAssistantSP> assistants =  m_d->doc->assistants();
     QMap<KisPaintingAssistantHandleSP, int> handlemap;
+
+    bool success = true;
     if (!assistants.isEmpty()) {
 
         Q_FOREACH (KisPaintingAssistantSP assist, assistants){
@@ -351,12 +485,18 @@ bool KisKraSaver::saveAssistants(KoStore* store, QString uri, bool external)
             location += QString(assist->id()+"%1.assistant").arg(assistantcounters[assist->id()]);
 
             data = assist->saveXml(handlemap);
-            store->open(location);
-            store->write(data);
-            store->close();
+            if (store->open(location)) {
+                qint64 nwritten = store->write(data);
+                bool r = store->close();
+                success = success && r && (nwritten == data.size());
+            } else {
+                success = false;
+            }
             assistantcounters[assist->id()]++;
         }
-
+    }
+    if (!success) {
+        m_d->errorMessages.append(i18nc("Saving .kra file error message", "Could not save assistants."));
     }
     return true;
 }
@@ -434,18 +574,26 @@ bool KisKraSaver::saveGuides(QDomDocument& doc, QDomElement& element)
     return true;
 }
 
+bool KisKraSaver::saveMirrorAxis(QDomDocument &doc, QDomElement &element)
+{
+    KisMirrorAxisConfig mirrorAxisConfig = m_d->doc->mirrorAxisConfig();
+
+    if (!mirrorAxisConfig.isDefault()) {
+        QDomElement mirrorAxisElement = mirrorAxisConfig.saveToXml(doc, MIRROR_AXIS);
+        element.appendChild(mirrorAxisElement);
+    }
+
+    return true;
+}
+
 bool KisKraSaver::saveAudio(QDomDocument& doc, QDomElement& element)
 {
     const KisImageAnimationInterface *interface = m_d->doc->image()->animationInterface();
     QString fileName = interface->audioChannelFileName();
+
     if (fileName.isEmpty()) return true;
 
-    if (!QFileInfo::exists(fileName)) {
-        m_d->errorMessages << i18n("Audio channel file %1 doesn't exist!", fileName);
-        return false;
-    }
-
-    const QDir documentDir = QFileInfo(m_d->doc->localFilePath()).absoluteDir();
+    const QDir documentDir = QFileInfo(m_d->filename).absoluteDir();
     KIS_ASSERT_RECOVER_RETURN_VALUE(documentDir.exists(), false);
 
     fileName = documentDir.relativeFilePath(fileName);
