@@ -21,6 +21,8 @@
 
 #include "SelectionDecorator.h"
 
+#include <QPainterPath>
+
 #include <KoShape.h>
 #include <KoSelection.h>
 #include <KoResourcePaths.h>
@@ -28,20 +30,27 @@
 
 #include "kis_debug.h"
 #include <KisHandlePainterHelper.h>
-#include <KoCanvasResourceManager.h>
+#include <KoCanvasResourceProvider.h>
 #include <KisQPainterStateSaver.h>
 #include "KoShapeGradientHandles.h"
+#include <KoCanvasBase.h>
+#include <KoSvgTextShape.h>
 
 #include "kis_painting_tweaks.h"
+#include "kis_coordinates_converter.h"
+#include "kis_icon_utils.h"
+
+
 
 #define HANDLE_DISTANCE 10
 
-SelectionDecorator::SelectionDecorator(KoCanvasResourceManager *resourceManager)
+SelectionDecorator::SelectionDecorator(KoCanvasResourceProvider *resourceManager)
     : m_hotPosition(KoFlake::Center)
     , m_handleRadius(7)
     , m_lineWidth(2)
     , m_showFillGradientHandles(false)
     , m_showStrokeFillGradientHandles(false)
+    , m_forceShapeOutlines(false)
 {
     m_hotPosition =
         KoFlake::AnchorPosition(
@@ -69,25 +78,57 @@ void SelectionDecorator::setShowStrokeFillGradientHandles(bool value)
     m_showStrokeFillGradientHandles = value;
 }
 
+void SelectionDecorator::setShowFillMeshGradientHandles(bool value)
+{
+    m_showFillMeshGradientHandles = value;
+}
+
+void SelectionDecorator::setCurrentMeshGradientHandles(const KoShapeMeshGradientHandles::Handle &selectedHandle,
+                                                       const KoShapeMeshGradientHandles::Handle &hoveredHandle)
+{
+    m_selectedMeshHandle = selectedHandle;
+    m_currentHoveredMeshHandle = hoveredHandle;
+}
+
 void SelectionDecorator::paint(QPainter &painter, const KoViewConverter &converter)
 {
-    const bool haveOnlyOneEditableShape = m_selection->selectedEditableShapes().size() == 1;
-
-    bool editable = false;
-
-    QList<KoShape*> selectedShapes = m_selection->selectedShapes();
+    QList<KoShape*> selectedShapes = m_selection->selectedVisibleShapes();
     if (selectedShapes.isEmpty()) return;
 
-    foreach (KoShape *shape, KoShape::linearizeSubtree(selectedShapes)) {
+    const bool haveOnlyOneEditableShape =
+        m_selection->selectedEditableShapes().size() == 1 &&
+        selectedShapes.size() == 1;
+
+    bool editable = false;
+    bool forceBoundngRubberLine = false;
+
+    Q_FOREACH (KoShape *shape, KoShape::linearizeSubtree(selectedShapes)) {
         if (!haveOnlyOneEditableShape || !m_showStrokeFillGradientHandles) {
             KisHandlePainterHelper helper =
-                KoShape::createHandlePainterHelper(&painter, shape, converter, m_handleRadius);
+                KoShape::createHandlePainterHelperView(&painter, shape, converter, m_handleRadius);
 
             helper.setHandleStyle(KisHandleStyle::secondarySelection());
-            helper.drawRubberLine(shape->outlineRect());
+
+            if (!m_forceShapeOutlines) {
+                helper.drawRubberLine(shape->outlineRect());
+            } else {
+                QList<QPolygonF> polys = shape->outline().toSubpathPolygons();
+
+                if (polys.size() == 1) {
+                    const QPolygonF poly1 = polys[0];
+                    const QPolygonF poly2 = QPolygonF(polys[0].boundingRect());
+                    const QPolygonF nonoverlap = poly2.subtracted(poly1);
+
+                    forceBoundngRubberLine |= !nonoverlap.isEmpty();
+                }
+
+                Q_FOREACH (const QPolygonF &poly, polys) {
+                    helper.drawRubberLine(poly);
+                }
+            }
         }
 
-        if (!shape->isGeometryProtected()) {
+        if (shape->isShapeEditable()) {
             editable = true;
         }
     }
@@ -95,9 +136,9 @@ void SelectionDecorator::paint(QPainter &painter, const KoViewConverter &convert
     const QRectF handleArea = m_selection->outlineRect();
 
     // draw extra rubber line around all the shapes
-    if (selectedShapes.size() > 1) {
+    if (selectedShapes.size() > 1 || forceBoundngRubberLine) {
         KisHandlePainterHelper helper =
-            KoShape::createHandlePainterHelper(&painter, m_selection, converter, m_handleRadius);
+            KoShape::createHandlePainterHelperView(&painter, m_selection, converter, m_handleRadius);
 
         helper.setHandleStyle(KisHandleStyle::primarySelection());
         helper.drawRubberLine(handleArea);
@@ -107,7 +148,7 @@ void SelectionDecorator::paint(QPainter &painter, const KoViewConverter &convert
     // is no need drawing the selection handles
     if (editable) {
         KisHandlePainterHelper helper =
-            KoShape::createHandlePainterHelper(&painter, m_selection, converter, m_handleRadius);
+            KoShape::createHandlePainterHelperView(&painter, m_selection, converter, m_handleRadius);
         helper.setHandleStyle(KisHandleStyle::primarySelection());
 
         QPolygonF outline = handleArea;
@@ -136,7 +177,13 @@ void SelectionDecorator::paint(QPainter &painter, const KoViewConverter &convert
         } else if (m_showStrokeFillGradientHandles) {
             paintGradientHandles(shape, KoFlake::StrokeFill, painter, converter);
         }
+
+        // paint meshgradient handles
+        if(m_showFillMeshGradientHandles) {
+            paintMeshGradientHandles(shape, KoFlake::Fill, painter, converter);
+        }
     }
+
 }
 
 void SelectionDecorator::paintGradientHandles(KoShape *shape, KoFlake::FillVariant fillVariant, QPainter &painter, const KoViewConverter &converter)
@@ -145,9 +192,9 @@ void SelectionDecorator::paintGradientHandles(KoShape *shape, KoFlake::FillVaria
     QVector<KoShapeGradientHandles::Handle> handles = gradientHandles.handles();
 
     KisHandlePainterHelper helper =
-        KoShape::createHandlePainterHelper(&painter, shape, converter, m_handleRadius);
+        KoShape::createHandlePainterHelperView(&painter, shape, converter, m_handleRadius);
 
-    const QTransform t = shape->absoluteTransformation(0).inverted();
+    const QTransform t = shape->absoluteTransformation().inverted();
 
     if (gradientHandles.type() == QGradient::LinearGradient) {
         KIS_SAFE_ASSERT_RECOVER_NOOP(handles.size() == 2);
@@ -168,3 +215,48 @@ void SelectionDecorator::paintGradientHandles(KoShape *shape, KoFlake::FillVaria
         }
     }
 }
+
+void SelectionDecorator::paintMeshGradientHandles(KoShape *shape,
+                                                  KoFlake::FillVariant fillVariant,
+                                                  QPainter &painter,
+                                                  const KoViewConverter &converter)
+{
+    KoShapeMeshGradientHandles gradientHandles(fillVariant, shape);
+
+    KisHandlePainterHelper helper =
+        KoShape::createHandlePainterHelperView(&painter, shape, converter, m_handleRadius);
+    helper.setHandleStyle(KisHandleStyle::secondarySelection());
+
+    helper.drawPath(gradientHandles.path());
+
+    // invert them, because we draw in logical coordinates.
+    QTransform t = shape->absoluteTransformation().inverted();
+    QVector<KoShapeMeshGradientHandles::Handle> cornerHandles = gradientHandles.handles();
+    for (const auto& corner: cornerHandles) {
+        if (corner.type == KoShapeMeshGradientHandles::Handle::BezierHandle) {
+            helper.drawHandleSmallCircle(t.map(corner.pos));
+        } else if (corner.type == KoShapeMeshGradientHandles::Handle::Corner) {
+            helper.drawHandleRect(t.map(corner.pos));
+        }
+    }
+
+    helper.setHandleStyle(KisHandleStyle::highlightedPrimaryHandlesWithSolidOutline());
+
+    // highlight the selected handle (only corner)
+    if (m_selectedMeshHandle.type == KoShapeMeshGradientHandles::Handle::Corner) {
+        helper.drawHandleRect(t.map(gradientHandles.getHandle(m_selectedMeshHandle.getPosition()).pos));
+    }
+
+    // highlight the path which is being hovered/moved
+    if (m_currentHoveredMeshHandle.type != KoShapeMeshGradientHandles::Handle::None) {
+        QVector<QPainterPath> paths = gradientHandles.getConnectedPath(m_currentHoveredMeshHandle);
+        for (const auto &path: paths) {
+            helper.drawPath(path);
+        }
+    }
+}
+
+void SelectionDecorator::setForceShapeOutlines(bool value)
+{
+    m_forceShapeOutlines = value;
+}
