@@ -35,7 +35,7 @@ KisToolSelectPath::KisToolSelectPath(KoCanvasBase * canvas)
     : KisToolSelectBase<KisDelegatedSelectPathWrapper>(canvas,
                                                        KisCursor::load("tool_polygonal_selection_cursor.png", 6, 6),
                                                        i18n("Select path"),
-                                                       (KisTool*) (new __KisToolSelectPathLocalTool(canvas, this)))
+                                                       new __KisToolSelectPathLocalTool(canvas, this))
 {
 }
 
@@ -49,12 +49,6 @@ void KisToolSelectPath::requestStrokeCancellation()
     localTool()->cancelPath();
 }
 
-void KisToolSelectPath::mousePressEvent(KoPointerEvent* event)
-{
-    if (!selectionEditable()) return;
-    DelegatedSelectPathTool::mousePressEvent(event);
-}
-
 // Install an event filter to catch right-click events.
 // This code is duplicated in kis_tool_path.cc
 bool KisToolSelectPath::eventFilter(QObject *obj, QEvent *event)
@@ -63,13 +57,13 @@ bool KisToolSelectPath::eventFilter(QObject *obj, QEvent *event)
     if (event->type() == QEvent::MouseButtonPress ||
             event->type() == QEvent::MouseButtonDblClick) {
         QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
-        if (mouseEvent->button() == Qt::RightButton) {
+        if (mouseEvent->button() == Qt::RightButton && hasUserInteractionRunning()) {
             localTool()->removeLastPoint();
             return true;
         }
     } else if (event->type() == QEvent::TabletPress) {
         QTabletEvent *tabletEvent = static_cast<QTabletEvent*>(event);
-        if (tabletEvent->button() == Qt::RightButton) {
+        if (tabletEvent->button() == Qt::RightButton && hasUserInteractionRunning()) {
             localTool()->removeLastPoint();
             return true;
         }
@@ -91,15 +85,55 @@ QList<QPointer<QWidget> > KisToolSelectPath::createOptionWidgets()
 }
 
 void KisDelegatedSelectPathWrapper::beginPrimaryAction(KoPointerEvent *event) {
-    mousePressEvent(event);
+    DelegatedSelectPathTool::mousePressEvent(event);
 }
 
 void KisDelegatedSelectPathWrapper::continuePrimaryAction(KoPointerEvent *event){
-    mouseMoveEvent(event);
+    DelegatedSelectPathTool::mouseMoveEvent(event);
 }
 
 void KisDelegatedSelectPathWrapper::endPrimaryAction(KoPointerEvent *event) {
-    mouseReleaseEvent(event);
+    DelegatedSelectPathTool::mouseReleaseEvent(event);
+}
+
+void KisDelegatedSelectPathWrapper::beginPrimaryDoubleClickAction(KoPointerEvent *event)
+{
+    DelegatedSelectPathTool::mouseDoubleClickEvent(event);
+}
+
+void KisDelegatedSelectPathWrapper::mousePressEvent(KoPointerEvent *event)
+{
+    // this event will be forwarded using beginPrimaryAction
+    Q_UNUSED(event);
+}
+
+void KisDelegatedSelectPathWrapper::mouseMoveEvent(KoPointerEvent *event)
+{
+    DelegatedSelectPathTool::mouseMoveEvent(event);
+
+    // WARNING: the code is duplicated from KisToolPaint::requestUpdateOutline
+    KisCanvas2 * kiscanvas = dynamic_cast<KisCanvas2*>(canvas());
+    KisPaintingAssistantsDecorationSP decoration = kiscanvas->paintingAssistantsDecoration();
+    if (decoration && decoration->visible() && decoration->hasPaintableAssistants()) {
+        kiscanvas->updateCanvas();
+    }
+}
+
+void KisDelegatedSelectPathWrapper::mouseReleaseEvent(KoPointerEvent *event)
+{
+    // this event will be forwarded using continuePrimaryAction
+    Q_UNUSED(event);
+}
+
+void KisDelegatedSelectPathWrapper::mouseDoubleClickEvent(KoPointerEvent *event)
+{
+    // this event will be forwarded using endPrimaryAction
+    Q_UNUSED(event);
+}
+
+bool KisDelegatedSelectPathWrapper::hasUserInteractionRunning() const
+{
+    return localTool()->pathStarted();
 }
 
 
@@ -135,7 +169,12 @@ void __KisToolSelectPathLocalTool::addPathShape(KoPathShape* pathShape)
 
     KisSelectionToolHelper helper(kisCanvas, kundo2_i18n("Select by Bezier Curve"));
 
-    if (m_selectionTool->selectionMode() == PIXEL_SELECTION) {
+    const SelectionMode mode =
+        helper.tryOverrideSelectionMode(kisCanvas->viewManager()->selection(),
+                                        m_selectionTool->selectionMode(),
+                                        m_selectionTool->selectionAction());
+
+    if (mode == PIXEL_SELECTION) {
 
         KisPixelSelectionSP tmpSel = KisPixelSelectionSP(new KisPixelSelection());
 
@@ -157,7 +196,18 @@ void __KisToolSelectPathLocalTool::addPathShape(KoPathShape* pathShape)
 
         delete pathShape;
     } else {
-        helper.addSelectionShape(pathShape);
+        helper.addSelectionShape(pathShape, m_selectionTool->selectionAction());
+    }
+}
+
+void KisToolSelectPath::resetCursorStyle()
+{
+    if (selectionAction() == SELECTION_ADD) {
+        useCursor(KisCursor::load("tool_polygonal_selection_cursor_add.png", 6, 6));
+    } else if (selectionAction() == SELECTION_SUBTRACT) {
+        useCursor(KisCursor::load("tool_polygonal_selection_cursor_sub.png", 6, 6));
+    } else {
+        KisToolSelectBase<KisDelegatedSelectPathWrapper>::resetCursorStyle();
     }
 }
 
