@@ -20,6 +20,7 @@
 
 #include <QPointF>
 #include <QPainter>
+#include <QPainterPath>
 #include <QMatrix4x4>
 #include <QVector2D>
 
@@ -52,7 +53,8 @@ struct KisPerspectiveTransformStrategy::Private
           converter(_converter),
           currentArgs(_currentArgs),
           transaction(_transaction),
-          imageTooBig(false)
+          imageTooBig(false),
+          isTransforming(false)
     {
     }
 
@@ -99,6 +101,7 @@ struct KisPerspectiveTransformStrategy::Private
 
     QPointF clickPos;
     ToolTransformArgs clickArgs;
+    bool isTransforming;
 
     QCursor getScaleCursor(const QPointF &handlePt);
     QCursor getShearCursor(const QPointF &start, const QPointF &end);
@@ -149,9 +152,11 @@ void KisPerspectiveTransformStrategy::Private::recalculateTransformedHandles()
     transformedHandles.yVanishing = v.toVector2DAffine().toPointF();
 }
 
-void KisPerspectiveTransformStrategy::setTransformFunction(const QPointF &mousePos, bool perspectiveModifierActive)
+void KisPerspectiveTransformStrategy::setTransformFunction(const QPointF &mousePos, bool perspectiveModifierActive, bool shiftModifierActive, bool altModifierActive)
 {
     Q_UNUSED(perspectiveModifierActive);
+    Q_UNUSED(shiftModifierActive);
+    Q_UNUSED(altModifierActive);
 
     QPolygonF transformedPolygon = m_d->transform.map(QPolygonF(m_d->transaction.originalRect()));
     StrokeFunction defaultFunction = transformedPolygon.containsPoint(mousePos, Qt::OddEvenFill) ? MOVE : NONE;
@@ -239,6 +244,10 @@ void KisPerspectiveTransformStrategy::paint(QPainter &gc)
 
     gc.save();
 
+    if (m_d->isTransforming) {
+        gc.setOpacity(0.1);
+    }
+
     /**
      * WARNING: we cannot install a transform to paint the handles here!
      *
@@ -458,6 +467,8 @@ void KisPerspectiveTransformStrategy::continuePrimaryAction(const QPointF &mouse
     Q_UNUSED(shiftModifierActve);
     Q_UNUSED(altModifierActive);
 
+    m_d->isTransforming = true;
+
     switch (m_d->function) {
     case NONE:
         break;
@@ -605,6 +616,7 @@ void KisPerspectiveTransformStrategy::continuePrimaryAction(const QPointF &mouse
 bool KisPerspectiveTransformStrategy::endPrimaryAction()
 {
     bool shouldSave = !m_d->imageTooBig;
+    m_d->isTransforming = false;
 
     if (m_d->imageTooBig) {
         m_d->currentArgs = m_d->clickArgs;
